@@ -13,6 +13,7 @@
 #
 # END HEADER
 
+import builtins
 import collections
 import datetime
 import decimal
@@ -32,9 +33,9 @@ from types import FunctionType
 
 from hypothesis import strategies as st
 from hypothesis.errors import InvalidArgument, ResolutionFailed
-from hypothesis.internal.compat import ForwardRef, typing_root_type
+from hypothesis.internal.compat import PYPY, ForwardRef, typing_root_type
 from hypothesis.internal.conjecture.utils import many as conjecture_utils_many
-from hypothesis.strategies._internal.datetime import zoneinfo
+from hypothesis.strategies._internal.datetime import zoneinfo  # type: ignore
 from hypothesis.strategies._internal.ipaddress import (
     SPECIAL_IPv4_RANGES,
     SPECIAL_IPv6_RANGES,
@@ -58,40 +59,92 @@ try:
 except ImportError:
     _GenericAlias = ()
 
+try:
+    from typing import _AnnotatedAlias  # type: ignore
+except ImportError:
+    try:
+        from typing_extensions import _AnnotatedAlias
+    except ImportError:
+        try:
+            from typing_extensions import (  # type: ignore
+                AnnotatedMeta as _AnnotatedAlias,
+            )
+
+            assert sys.version_info[:2] == (3, 6)
+        except ImportError:
+            _AnnotatedAlias = ()
+
 
 def type_sorting_key(t):
     """Minimise to None, then non-container types, then container types."""
-    if not is_a_type(t):  # This branch is for Python < 3.8
-        raise InvalidArgument("thing=%s must be a type" % (t,))  # pragma: no cover
+    if not (is_a_type(t) or is_typing_literal(t)):  # This branch is for Python < 3.8
+        raise InvalidArgument(f"thing={t} must be a type")  # pragma: no cover
     if t is None or t is type(None):  # noqa: E721
         return (-1, repr(t))
+    t = getattr(t, "__origin__", t)
     if not isinstance(t, type):  # pragma: no cover
         # Some generics in the typing module are not actually types in 3.7
         return (2, repr(t))
     return (int(issubclass(t, collections.abc.Container)), repr(t))
 
 
+def _compatible_args(args, superclass_args):
+    """Check that the args of two generic types are compatible for try_issubclass."""
+    assert superclass_args is not None
+    if args is None:
+        return True
+    return len(args) == len(superclass_args) and all(
+        # "a==b or either is a typevar" is a hacky approximation, but it's
+        # good enough for all the cases that I've seen so far and has the
+        # substantial virtue of (relative) simplicity.
+        a == b or isinstance(a, typing.TypeVar) or isinstance(b, typing.TypeVar)
+        for a, b in zip(args, superclass_args)
+    )
+
+
 def try_issubclass(thing, superclass):
-    thing = getattr(thing, "__origin__", None) or thing
-    superclass = getattr(superclass, "__origin__", None) or superclass
     try:
-        return issubclass(thing, superclass)
+        # In this case we're looking at two distinct classes - which might be generics.
+        # That brings in some complications:
+        if issubclass(
+            getattr(thing, "__origin__", None) or thing,
+            getattr(superclass, "__origin__", None) or superclass,
+        ):
+            superclass_args = getattr(superclass, "__args__", None)
+            if sys.version_info[:2] == (3, 6) or not superclass_args:
+                # Python 3.6 doesn't have PEP-560 semantics or __orig_bases__,
+                # so there's no point continuing; we therefore stop early.
+                # The superclass is not generic, so we're definitely a subclass.
+                return True
+            # Sadly this is just some really fiddly logic to handle all the cases
+            # of user-defined generic types, types inheriting from parametrised
+            # generics, and so on.  If you need to change this code, read PEP-560
+            # and Hypothesis issue #2951 closely first, and good luck.  The tests
+            # will help you, I hope - good luck.
+            if getattr(thing, "__args__", None) is not None:
+                return True
+            for orig_base in getattr(thing, "__orig_bases__", None) or [None]:
+                args = getattr(orig_base, "__args__", None)
+                if _compatible_args(args, superclass_args):
+                    return True
+        return False
     except (AttributeError, TypeError):
         # Some types can't be the subject or object of an instance or subclass check
         return False
 
 
 def is_a_new_type(thing):
-    # At runtime, `typing.NewType` returns an identity function rather
-    # than an actual type, but we can check whether that thing matches.
-    return (
-        hasattr(thing, "__supertype__")
-        and (
-            getattr(thing, "__module__", None) == "typing"
-            or getattr(thing, "__module__", None) == "typing_extensions"
+    if not isinstance(typing.NewType, type):
+        # At runtime, `typing.NewType` returns an identity function rather
+        # than an actual type, but we can check whether that thing matches.
+        return (
+            hasattr(thing, "__supertype__")
+            and getattr(thing, "__module__", None) in ("typing", "typing_extensions")
+            and inspect.isfunction(thing)
         )
-        and inspect.isfunction(thing)
-    )
+    # In 3.10 and later, NewType is actually a class - which simplifies things.
+    # See https://bugs.python.org/issue44353 for links to the various patches.
+    return isinstance(thing, typing.NewType)  # pragma: no cover  # on 3.8, anyway
 
 
 def is_a_type(thing):
@@ -104,10 +157,36 @@ def is_typing_literal(thing):
         hasattr(typing, "Literal")
         and getattr(thing, "__origin__", None) == typing.Literal
         or hasattr(typing_extensions, "Literal")
-        and getattr(thing, "__origin__", None) == typing_extensions.Literal
+        and (
+            getattr(thing, "__origin__", None) == typing_extensions.Literal
+            or sys.version_info[:2] == (3, 6)
+            and isinstance(thing, type(typing_extensions.Literal[None]))
+        )
     )
 
 
+def is_annotated_type(thing):
+    return (
+        isinstance(thing, _AnnotatedAlias)
+        and getattr(thing, "__args__", None) is not None
+    )
+
+
+def find_annotated_strategy(annotated_type):  # pragma: no cover
+    flattened_meta = []
+
+    all_args = (
+        *getattr(annotated_type, "__args__", ()),
+        *getattr(annotated_type, "__metadata__", ()),
+    )
+    for arg in all_args:
+        if is_annotated_type(arg):
+            flattened_meta.append(find_annotated_strategy(arg))
+        if isinstance(arg, st.SearchStrategy):
+            flattened_meta.append(arg)
+    return flattened_meta[-1] if flattened_meta else None
+
+
 def has_type_arguments(type_):
     """Decides whethere or not this type has applied type arguments."""
     args = getattr(type_, "__args__", None)
@@ -121,9 +200,9 @@ def has_type_arguments(type_):
 
 
 def is_generic_type(type_):
-    """Decides whethere a given type is generic or not."""
+    """Decides whether a given type is generic or not."""
     # The ugly truth is that `MyClass`, `MyClass[T]`, and `MyClass[int]` are very different.
-    # In different python versions the might have the same type (3.6)
+    # In different python versions they might have the same type (3.6)
     # or it can be regular type vs `_GenericAlias` (3.7+)
     # We check for `MyClass[T]` and `MyClass[int]` with the first condition,
     # while the second condition is for `MyClass` in `python3.7+`.
@@ -193,11 +272,20 @@ def from_typing_type(thing):
             else:
                 literals.append(arg)
         return st.sampled_from(literals)
+    if is_annotated_type(thing):  # pragma: no cover
+        # This requires Python 3.9+ or the typing_extensions package
+        annotated_strategy = find_annotated_strategy(thing)
+        if annotated_strategy is not None:
+            return annotated_strategy
+        args = thing.__args__
+        assert args, "it's impossible to make an annotated type with no args"
+        annotated_type = args[0]
+        return st.from_type(annotated_type)
     # Now, confirm that we're dealing with a generic type as we expected
     if sys.version_info[:2] < (3, 9) and not isinstance(
         thing, typing_root_type
     ):  # pragma: no cover
-        raise ResolutionFailed("Cannot resolve %s to a strategy" % (thing,))
+        raise ResolutionFailed(f"Cannot resolve {thing} to a strategy")
 
     # Some "generic" classes are not generic *in* anything - for example both
     # Hashable and Sized have `__args__ == ()` on Python 3.7 or later.
@@ -249,6 +337,12 @@ def from_typing_type(thing):
             for T in list(union_elems) + [elem_type]
         ):
             mapping.pop(typing.ByteString, None)
+    elif (
+        (not mapping)
+        and isinstance(thing, ForwardRef)
+        and thing.__forward_arg__ in vars(builtins)
+    ):
+        return st.from_type(getattr(builtins, thing.__forward_arg__))
     strategies = [
         v if isinstance(v, st.SearchStrategy) else v(thing)
         for k, v in mapping.items()
@@ -257,8 +351,8 @@ def from_typing_type(thing):
     empty = ", ".join(repr(s) for s in strategies if s.is_empty)
     if empty or not strategies:
         raise ResolutionFailed(
-            "Could not resolve %s to a strategy; consider using "
-            "register_type_strategy" % (empty or thing,)
+            f"Could not resolve {empty or thing} to a strategy; "
+            "consider using register_type_strategy"
         )
     return st.one_of(strategies)
 
@@ -290,7 +384,9 @@ utc_offsets = st.builds(
 # As a general rule, we try to limit this to scalars because from_type()
 # would have to decide on arbitrary collection elements, and we'd rather
 # not (with typing module generic types and some builtins as exceptions).
-_global_type_lookup = {
+_global_type_lookup: typing.Dict[
+    type, typing.Union[st.SearchStrategy, typing.Callable[[type], st.SearchStrategy]]
+] = {
     type(None): st.none(),
     bool: st.booleans(),
     int: st.integers(),
@@ -330,7 +426,7 @@ _global_type_lookup = {
         st.none() | st.integers(),
     ),
     range: st.one_of(
-        st.integers(min_value=0).map(range),
+        st.builds(range, st.integers(min_value=0)),
         st.builds(range, st.integers(), st.integers()),
         st.builds(range, st.integers(), st.integers(), st.integers().filter(bool)),
     ),
@@ -347,10 +443,54 @@ _global_type_lookup = {
         st.sampled_from(SPECIAL_IPv6_RANGES).map(ipaddress.IPv6Network),
     ),
     os.PathLike: st.builds(PurePath, st.text()),
+    UnicodeDecodeError: st.builds(
+        UnicodeDecodeError,
+        st.just("unknown encoding"),
+        st.just(b""),
+        st.just(0),
+        st.just(0),
+        st.just("reason"),
+    ),
+    UnicodeEncodeError: st.builds(
+        UnicodeEncodeError,
+        st.just("unknown encoding"),
+        st.text(),
+        st.just(0),
+        st.just(0),
+        st.just("reason"),
+    ),
+    UnicodeTranslateError: st.builds(
+        UnicodeTranslateError, st.text(), st.just(0), st.just(0), st.just("reason")
+    ),
+    enumerate: st.builds(enumerate, st.just(())),
+    filter: st.builds(filter, st.just(lambda _: None), st.just(())),
+    map: st.builds(map, st.just(lambda _: None), st.just(())),
+    reversed: st.builds(reversed, st.just(())),
+    classmethod: st.builds(classmethod, st.just(lambda self: self)),
+    staticmethod: st.builds(staticmethod, st.just(lambda self: self)),
+    super: st.builds(super, st.from_type(type)),
     # Pull requests with more types welcome!
 }
 if zoneinfo is not None:  # pragma: no branch
     _global_type_lookup[zoneinfo.ZoneInfo] = st.timezones()
+if PYPY:
+    _global_type_lookup[builtins.sequenceiterator] = st.builds(iter, st.tuples())  # type: ignore
+try:
+    BaseExceptionGroup  # type: ignore # noqa
+except NameError:
+    pass
+else:  # pragma: no cover
+    _global_type_lookup[BaseExceptionGroup] = st.builds(  # type: ignore
+        BaseExceptionGroup,  # type: ignore
+        st.text(),
+        st.lists(st.from_type(BaseException), min_size=1),
+    )
+    _global_type_lookup[ExceptionGroup] = st.builds(  # type: ignore
+        ExceptionGroup,  # type: ignore
+        st.text(),
+        st.lists(st.from_type(Exception), min_size=1),
+    )
+
 
 _global_type_lookup[type] = st.sampled_from(
     [type(None)] + sorted(_global_type_lookup, key=str)
@@ -476,13 +616,16 @@ def resolve_Type(thing):
     if getattr(args[0], "__origin__", None) is typing.Union:
         args = args[0].__args__
     # Duplicate check from from_type here - only paying when needed.
-    for a in args:  # pragma: no cover  # only on Python 3.6
+    args = list(args)
+    for i, a in enumerate(args):
         if type(a) == ForwardRef:
-            raise ResolutionFailed(
-                "thing=%s cannot be resolved.  Upgrading to "
-                "python>=3.6 may fix this problem via improvements "
-                "to the typing module." % (thing,)
-            )
+            try:
+                args[i] = getattr(builtins, a.__forward_arg__)
+            except AttributeError:
+                raise ResolutionFailed(
+                    f"Cannot find the type referenced by {thing} - try using "
+                    f"st.register_type_strategy({thing}, st.from_type(...))"
+                ) from None
     return st.sampled_from(sorted(args, key=type_sorting_key))
 
 
@@ -651,7 +794,7 @@ def resolve_Callable(thing):
 
 @register(typing.TypeVar)
 def resolve_TypeVar(thing):
-    type_var_key = "typevar=%r" % (thing,)
+    type_var_key = f"typevar={thing!r}"
 
     if getattr(thing, "__bound__", None) is not None:
         bound = thing.__bound__
